#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "stdio.h"
#include "stdlib.h"
#include "math.h"

#include "jpeglib.h"	// jpeg support
#include <jerror.h>
#include <setjmp.h>	// jpeg error recovery mechanism
#include "plascolin.h"

#include <sys/dir.h>	// directory content handling
#include "sys/stat.h"	// for mkdir



/******************************************************
 * Global 
 *
 *****************************************************/
struct RGBImage img_r;		// right    image
struct RGBImage img_l;		// left     image
struct RGBImage img_a;		// anaglyph image

int x_offset;			// x_offset of the two stereo images
int y_offset;			// y_offset of the two stereo images


char workingdir[200];		/* name of working directory */
struct dirent **filelist;	// List of jpeg files in current workingdir
int imgnum;			// the number of leftimage in the list of jpeg files in current workingdir
unsigned int maxfiles;		// the maximum number of jpeg files in current workingdir
char leftfile[200];  		// the name of the left file
char rightfile[200];  		// the name of the right file

extern GtkWidget *Main_window;	/* -> declared in main.c     */
extern GtkWidget *Display2;
extern GtkWidget *Display3;


double disp_zoom = 1;		/* Display zoom value        */
char disp_measure_modus = 1;	/* Rangemodus 0=hf units     */
                                /*            1=pov units    */
int disp_offset_x = 0;		/* Display offset X value    */ 
int disp_offset_y = 0;		/* Display offset Y value    */ 

char disp2_on = 0;		/* Flag: wheather to show display2 window */
char disp3_on = 0;		/* Flag: wheather to show display3 window */
char disp2_hmirror_on = 0;	/* Flag: wheather to mirror display2*/
char disp3_hmirror_on = 0;	/* Flag: wheather to mirror display3 */


// The struct definition for color values 
struct rgb{
 unsigned char red;
 unsigned char green;
 unsigned char blue;
};




/******************************************************
 *  Function:  void display(void)
 *
 *  Description: 
 *
 *  Draw the image to the drawing area. 
 *
 *  Using GtkRGB (see http://developer.gnome.org/doc/API/2.0/gdk/gdk-GdkRGB.html#id2597033)
 *
*/
void display(void)
{
  GtkWidget *drawingarea = lookup_widget(GTK_WIDGET(Main_window),"drawingarea1");
  GtkRange *hscrollbar = (GtkRange*)lookup_widget(GTK_WIDGET(Main_window),"hscrollbar1");
  GtkRange *vscrollbar = (GtkRange*)lookup_widget(GTK_WIDGET(Main_window),"vscrollbar1");
  char tmp[200];	// a temporary string

  GdkGC *gc;
  GtkAdjustment *hadjust;
  GtkAdjustment *vadjust;
  unsigned char *buffer; 	// the buffer to hold the raw rgb backup to be displayed in drawable
  unsigned short int hf_value;	// the height value at a certain point in the heightfield
  struct rgb col;		// a rgb color triplet
  unsigned int x, y;
  unsigned int tx, ty;
  int width, height;


  // set size of the drawingarea widget
  width  = drawingarea->allocation.width;
  height = drawingarea->allocation.height;

  buffer = malloc( 3 * width * height * sizeof(char) );   // allocate some memory for the buffer 

  // get scrollbar values
  hadjust = gtk_range_get_adjustment(hscrollbar);
  vadjust = gtk_range_get_adjustment(vscrollbar);

  // Set horiz scrollbar values
  hadjust->value = disp_offset_x;
  hadjust->lower = 0;
  hadjust->upper = img_a.width * disp_zoom;
  hadjust->step_increment = 1;
  hadjust->page_increment = 10;
  hadjust->page_size = width;
  // Set vert scrollbar values
  vadjust->value = disp_offset_y;
  vadjust->lower = 0;
  vadjust->upper = img_a.height * disp_zoom;
  vadjust->step_increment = 1;
  vadjust->page_increment = 10;
  vadjust->page_size = height;

  gtk_range_set_adjustment(hscrollbar,hadjust);	// put new values to adjustment
  gtk_range_set_adjustment(vscrollbar,vadjust); //  :
  gtk_signal_emit_by_name (hadjust, "changed"); // to redraw the scrollbars with changed adjustment
  gtk_signal_emit_by_name (vadjust, "changed"); //  :


  gdk_rgb_init();  // init GdkRGB
  for(x=0; x<width; x++)
   for(y=0; y<height; y++)
   {
     tx=(x+disp_offset_x)/disp_zoom;		// Combine with disp_offset and disp_zoom
     ty=(y+disp_offset_y)/disp_zoom;		// Combine with disp_offset and disp_zoom

     if(tx<img_a.width && ty<img_a.height)		// inside hf area?
     {
        *(buffer + y*3*width + x*3 + 0 ) = *(img_a.data + ((ty*img_a.width) + tx) * 3 + 0 );
        *(buffer + y*3*width + x*3 + 1 ) = *(img_a.data + ((ty*img_a.width) + tx) * 3 + 1 );  
        *(buffer + y*3*width + x*3 + 2 ) = *(img_a.data + ((ty*img_a.width) + tx) * 3 + 2 );  
     }
     else 				    // Point is outside of hf -> BLACK PIXEL
     {
        *(buffer + y*3*width + x*3 + 0 ) = 0;
        *(buffer + y*3*width + x*3 + 1 ) = 0;  
        *(buffer + y*3*width + x*3 + 2 ) = 0;  
     }   

   }  
  gc = gdk_gc_new(drawingarea->window);
  gdk_draw_rgb_image (  drawingarea->window, gc, 0, 0, width, height,
                        GDK_RGB_DITHER_NONE, buffer, width*3 );
  free(buffer);	

  // Show information about anaglyph on label in statusbar
  sprintf(tmp,"%2.2f%c  %ix%i",100*disp_zoom,37, img_a.width,img_a.height);
  gtk_label_set_text (lookup_widget(GTK_WIDGET(Main_window),"lb_anaglyph"), tmp );
}




/******************************************************
 *  Function:  void display2(void)
 *
 *  Description: 
 *
 *  Display2 - Draw the image to the drawing area. 
 *
*/
void display2(void)
{
  GtkWidget *drawingarea = lookup_widget(GTK_WIDGET(Display2),"drawingarea2");
  char tmp[200];	// a temporary string

  GdkGC *gc;
  unsigned char *buffer; 	// the buffer to hold the raw rgb backup to be displayed in drawable
  unsigned short int hf_value;	// the height value at a certain point in the heightfield
  struct rgb col;		// a rgb color triplet
  unsigned int x, y;
  unsigned int tx, ty;
  int width, height;
  double zoom = 1;		/* Display zoom value        */

  // set size of the drawingarea widget
  width  = drawingarea->allocation.width;
  height = drawingarea->allocation.height;

  buffer = malloc( 3 * width * height * sizeof(char) );   // allocate some memory for the buffer 

  // Calculate the appropriate zoom factor so that the image matches the drawable
  if(width > height)
  {
   if((double)img_a.width/img_a.height < (double)width/height)	
     zoom=(double)height/img_a.height;
   else
     zoom=(double)width/img_a.width;
  }
  else
  {
   if((double)img_a.width/img_a.height > (double)width/height)	
     zoom=(double)width/img_a.width;
   else				
     zoom=(double)height/img_a.height;
  }


  gdk_rgb_init();  // init GdkRGB

  for(x=0; x<width; x++)
   for(y=0; y<height; y++)
   {
      tx= x / zoom ;		// Combine with zoom
      ty= y / zoom ;		// Combine with zoom
      tx= tx- x_offset;
      ty= ty- y_offset;   

      if( tx<img_l.width && ty<img_l.height  )		// inside hf area?
      {
       if(disp2_hmirror_on==0)
       {
         *(buffer + y*3*width + x*3 + 0 ) = *( img_l.data + ((ty*img_l.width) + tx) * 3 + 0 );
         *(buffer + y*3*width + x*3 + 1 ) = *( img_l.data + ((ty*img_l.width) + tx) * 3 + 1 );  
         *(buffer + y*3*width + x*3 + 2 ) = *( img_l.data + ((ty*img_l.width) + tx) * 3 + 2 );  
       }
       else
       {
         *(buffer + y*3*width + x*3 + 0 ) = *( img_l.data + ((ty*img_l.width) + img_l.width-tx) * 3 + 0 );
         *(buffer + y*3*width + x*3 + 1 ) = *( img_l.data + ((ty*img_l.width) + img_l.width-tx) * 3 + 1 );  
         *(buffer + y*3*width + x*3 + 2 ) = *( img_l.data + ((ty*img_l.width) + img_l.width-tx) * 3 + 2 );  
       }
      }
      else 				    // Point is outside of hf -> BLACK PIXEL
      {
         *(buffer + y*3*width + x*3 + 0 ) = 0;
         *(buffer + y*3*width + x*3 + 1 ) = 0;  
         *(buffer + y*3*width + x*3 + 2 ) = 0;  
      }   
   }  

  gc = gdk_gc_new(drawingarea->window);
  gdk_draw_rgb_image (  drawingarea->window, gc, 0, 0, width, height,
                        GDK_RGB_DITHER_NONE, buffer, width*3 );
  free(buffer);	
}




/******************************************************
 *  Function:  void display3(void)
 *
 *  Description: 
 *
 *  Display3 - Draw the image to the drawing area. 
 *
*/
void display3(void)
{
  GtkWidget *drawingarea = lookup_widget(GTK_WIDGET(Display3),"drawingarea3");
  char tmp[200];	// a temporary string

  GdkGC *gc;
  unsigned char *buffer; 	// the buffer to hold the raw rgb backup to be displayed in drawable
  unsigned short int hf_value;	// the height value at a certain point in the heightfield
  struct rgb col;		// a rgb color triplet
  unsigned int x, y;
  unsigned int tx, ty;
  int width, height;
  double zoom = 1;		/* Display zoom value        */

  // set size of the drawingarea widget
  width  = drawingarea->allocation.width;
  height = drawingarea->allocation.height;

  buffer = malloc( 3 * width * height * sizeof(char) );   // allocate some memory for the buffer 

  // Calculate the appropriate zoom factor so that the image matches the drawable
  if(width > height)
  {
   if((double)img_a.width/img_a.height < (double)width/height)	
     zoom=(double)height/img_a.height;
   else
     zoom=(double)width/img_a.width;
  }
  else
  {
   if((double)img_a.width/img_a.height > (double)width/height)	
     zoom=(double)width/img_a.width;
   else				
     zoom=(double)height/img_a.height;
  }


  gdk_rgb_init();  // init GdkRGB
  for(x=0; x<width; x++)
   for(y=0; y<height; y++)
   {
     tx= x / zoom ;		// Combine with zoom
     ty= y / zoom ;		// Combine with zoom

     if( tx<img_a.width && ty<img_a.height   )		// inside hf area?
     {
       if(disp3_hmirror_on==0)
       {
        *(buffer + y*3*width + x*3 + 0 ) = *(img_r.data + ((ty*img_r.width) + tx) * 3 + 0 );
        *(buffer + y*3*width + x*3 + 1 ) = *(img_r.data + ((ty*img_r.width) + tx) * 3 + 1 );  
        *(buffer + y*3*width + x*3 + 2 ) = *(img_r.data + ((ty*img_r.width) + tx) * 3 + 2 );  
       }
       else
       {
        *(buffer + y*3*width + x*3 + 0 ) = *(img_r.data + ((ty*img_r.width) + img_r.width-tx) * 3 + 0 );
        *(buffer + y*3*width + x*3 + 1 ) = *(img_r.data + ((ty*img_r.width) + img_r.width-tx) * 3 + 1 );  
        *(buffer + y*3*width + x*3 + 2 ) = *(img_r.data + ((ty*img_r.width) + img_r.width-tx) * 3 + 2 );  
       }
     }
     else 				    // Point is outside of hf -> BLACK PIXEL
     {
        *(buffer + y*3*width + x*3 + 0 ) = 0;
        *(buffer + y*3*width + x*3 + 1 ) = 0;  
        *(buffer + y*3*width + x*3 + 2 ) = 0;  
     }   

   }  
  gc = gdk_gc_new(drawingarea->window);
  gdk_draw_rgb_image (  drawingarea->window, gc, 0, 0, width, height,
                        GDK_RGB_DITHER_NONE, buffer, width*3 );
  free(buffer);	
}






/******************************************************
 *  Function:  void zoomtofit(void);
 *
 *  Description: 
 *  - Compute a matching 'disp_zoom' value so that the image fits into the display 
 *   
 *
 *
*/
void zoomtofit(void)
{
  GtkWidget *drawingarea = lookup_widget(GTK_WIDGET(Main_window),"drawingarea1");
  int dr_width, dr_height;

  // get size of the drawingarea widget
  dr_width  = drawingarea->allocation.width;
  dr_height = drawingarea->allocation.height;

  if(dr_width > dr_height)
  {
   if((double)img_a.width/img_a.height < (double)dr_width/dr_height)	
     disp_zoom=(double)dr_height/img_a.height;
   else
     disp_zoom=(double)dr_width/img_a.width;
  }
  else
  {
   if((double)img_a.width/img_a.height > (double)dr_width/dr_height)	
     disp_zoom=(double)dr_width/img_a.width;
   else				
     disp_zoom=(double)dr_height/img_a.height;
  }

  disp_offset_x = 0;		 
  disp_offset_y = 0;		 
  display();
}





/******************************************************
 *  Function:  void write_config_file(void);
 *
 *  Description: 
 *  - Write a config file containing all information to be stored for next program start
 *  
*/
void write_config_file(void)
{
 char tmp[200];
 FILE *out;
 
 sprintf(tmp,"%s/.plascolin",getenv("HOME"));
 if((out = fopen(tmp,"w"))==NULL)
 {
   fprintf(out,"Unable to open config file for writing!\n");
   exit(0);
 }
 fprintf(out,"# plascolin config file\n");
 fprintf(out,"\n");
 fprintf(out,"width=%i\n",Main_window->allocation.width);
 fprintf(out,"height=%i\n",Main_window->allocation.height);
 fprintf(out,"\n");
 fprintf(out,"workingdir=%s\n",workingdir);
 fprintf(out,"\n");
 fprintf(out,"x_offset=%i\n",x_offset);
 fprintf(out,"y_offset=%i\n",y_offset);
 fprintf(out,"\n");
 fprintf(out,"disp2_on=%i\n",disp2_on);
 fprintf(out,"disp3_on=%i\n",disp3_on);
 fprintf(out,"\n");
 fprintf(out,"disp2_hmirror_on=%i\n",disp2_hmirror_on);
 fprintf(out,"disp3_hmirror_on=%i\n",disp3_hmirror_on);
 fprintf(out,"\n");

 fclose(out);
}




/******************************************************
 *  Function:  void read_config_file(void);
 *
 *  Description: 
 *  - If configfile exists read it
 *  - If it does not exist set default values
 *
*/
void read_config_file(void)
{
 char tmp[200];
 char token[200];
 char value[200];
 unsigned int width, height;	// temporary store width and height

 FILE *in;
 unsigned int t;
 


 sprintf(tmp,"%s/.plascolin",getenv("HOME"));
 if( (in = fopen(tmp,"r")) == NULL)
 {
   printf("Unable to open config file: %s\n",tmp);	// maybe there is no config file, first run?
   printf("Setting defaults\n");			// set defaults
   width=800;					
   height=600;
   sprintf(workingdir,"%s",getenv("PWD"));		// actual PWD
 }
 else						// can open config file
 {
  while (fgets(tmp, 100, in)) 
  {
   strcpy(token,tmp);
   token[strcspn(tmp,"=")]=0;  

   strcpy( value , tmp + strcspn(tmp,"=") + 1 );

   if(strcmp(token,"width")==0)
   {
     width=atoi(value);
   }
   if(strcmp(token,"height")==0)
   {
     height=atoi(value);
   }
   if(strcmp(token,"workingdir")==0)
   {
     value[strlen(value)-1]=0;
     strcpy(workingdir,value);
   }
   if(strcmp(token,"x_offset")==0)  
   {
     x_offset=atoi(value);
   }
   if(strcmp(token,"y_offset")==0)
   {
     y_offset=atoi(value);
   }
   if(strcmp(token,"disp2_on")==0)
   {
     disp2_on=atoi(value);
   }
   if(strcmp(token,"disp3_on")==0)
   {
     disp3_on=atoi(value);
   }
   if(strcmp(token,"disp2_hmirror_on")==0)
   {
     disp2_hmirror_on=atoi(value);
   }
   if(strcmp(token,"disp3_hmirror_on")==0)
   {
     disp3_hmirror_on=atoi(value);
   }

 }
  fclose(in);
 }


 // set window size to  X-Y
 gtk_window_resize (Main_window, width, height);


}




/******************************************************
 *  Function:  void open_files(void);
 *
 *  Description: 
 *  - open a gtk filecooser 
 *  - set filter for jpeg, jpg, ... 
 *
*/
void open_files(void)
{
 GtkWidget *open;		// the open dialog
 GtkFileFilter *filter_jpg;	// a filter (i.e.: ".jpeg")
 GtkFileFilter *filter_all;	// a filter (i.e.: ".jpeg")

 open = create_FileOpen();   	// Create the window 

 gtk_file_chooser_set_current_folder(open, workingdir);		// set the directory


 // Filter JPEG
 filter_jpg = gtk_file_filter_new ();			// Create a new GtkFileFilter with no rules
 gtk_file_filter_set_name(filter_jpg, "JPEG image (*.jpg *.jpeg *.JPG *.JPEG)");
							// Set the human-readable name of the filter.	
 gtk_file_filter_add_pattern(filter_jpg, "*.jpg" );
 gtk_file_filter_add_pattern(filter_jpg, "*.jpeg");
 gtk_file_filter_add_pattern(filter_jpg, "*.JPG" );
 gtk_file_filter_add_pattern(filter_jpg, "*.JPEG");
 gtk_file_chooser_add_filter(open, filter_jpg );	// add a filter (i.e.: ".jpeg")

 // Filter All
 filter_all = gtk_file_filter_new ();			// Create a new GtkFileFilter with no rules
 gtk_file_filter_set_name(filter_all, "All Files");	// Set the human-readable name of the filter.
 gtk_file_filter_add_pattern(filter_all, "*" );
 gtk_file_chooser_add_filter(open, filter_all );	// add a filter (i.e.: ".jpeg")


 gtk_file_chooser_set_filter(open, filter_jpg );	// set filter_jpg
 gtk_widget_show(open);		// Display the window
}



/******************************************************
 *  Function:  void write_file(void);
 *
 *  Description: 
 *  - if not exist create a new subfolder 'stereos' in actual workingdir
 *  - write the actual 3D image file in subfolder 'stereos'
 *  -
 *
*/
void write_file(void)
{
 char tmp[200];

 if(img_a.data==0)
  printf("No file loaded!\n");
 else
 {
  sprintf(tmp,"%s/stereos",workingdir);

  mkdir (tmp, S_IRWXU);		// create directory

  sprintf(tmp,"%s/stereos/%s",workingdir,leftfile);
  printf(": %s \n",tmp);
  write_JPEG_file(tmp, 95, img_a.data, img_a.width, img_a.height );
 }
}





/******************************************************
 *  Function:  void exit_appl(void);
 *
 *  Description: 
 *  - write data to config file etc
 *  - exits the main application 
 *
*/
void exit_appl(void)
{
 write_config_file();		// write configuration data before exiting
 gtk_main_quit();
}




/******************************************************
 *  Function:  	void zoom_in(void);
 *		void zoom_out(void);
 *
 *  Description: 
 *  -  
 *
*/
void zoom_in(void)
{
 if(disp_zoom<16) disp_zoom=disp_zoom*1.1;	// limit zoom factor
 display();
}

void zoom_out(void)
{
 if(disp_zoom>0.015625) disp_zoom=disp_zoom/1.1;	// limit zoom factor
 display();
}




/******************************************************
 *  Function:  	void create_anaglyph(void);
 *
 *  Description: 
 *  - This function creates the anaglyph image buffer
 *    and redraws it on the display   
 *
*/
void create_anaglyph(void)
{
 unsigned int ax, ay;
 int lx, ly, rx, ry;

 if(img_a.data!=0)   free(img_a.data);	// free buffer if it was used before
 img_a.width  = img_l.width -abs(x_offset);
 img_a.height = img_l.height-abs(y_offset);
 //printf("Image size: %i x %i\n",img_a.width,img_a.height);
 img_a.data = malloc( 3 * img_a.width * img_a.height * sizeof(char) );   // allocate some memory for the buffer 

 for(ay=0; ay<img_a.height; ay++)
  for(ax=0; ax<img_a.width; ax++)
  {
    lx=ax - x_offset;
    ly=ay - y_offset;
    rx=ax;
    ry=ay;

    if(lx<img_l.width && ly<img_l.height && rx<img_r.width && ry<img_r.height)
    {
     // RED-CYAN
     *(img_a.data + ((ax*3)+(ay*img_a.width*3)+0))   = *(img_l.data + ((lx*3)+(ly*img_l.width*3)+0));  // RED 
     *(img_a.data + ((ax*3)+(ay*img_a.width*3)+1))   = *(img_r.data + ((rx*3)+(ry*img_r.width*3)+1));  // GREEN
     *(img_a.data + ((ax*3)+(ay*img_a.width*3)+2))   = *(img_r.data + ((rx*3)+(ry*img_r.width*3)+2));  // BLUE
/*
     // ColorCode3D (Reversed)
     *(img_a.data + ((ax*3)+(ay*img_a.width*3)+0))   = *(img_r.data + ((rx*3)+(ry*img_r.width*3)+0));  // RED   
     *(img_a.data + ((ax*3)+(ay*img_a.width*3)+1))   = *(img_r.data + ((rx*3)+(ry*img_r.width*3)+1));  // GREEN
     *(img_a.data + ((ax*3)+(ay*img_a.width*3)+2))   = *(img_l.data + ((lx*3)+(ly*img_l.width*3)+2));  // BLUE
*/
    } 
 }

 //zoomtofit();
 display();
}


/******************************************************
 *  Function:  	void next_pair(void);
 *
 *  Description: 
 *  - load the next image pair 
 *      
*/
void next_pair(void)
{
 unsigned int x;
 char tmp2[200];	// a temporary string
 char tmp3[200];	// a temporary string

 if(filelist==0) 
   printf("No Image loaded \n");
 else
 {
  imgnum=imgnum+2;
  if(imgnum>=maxfiles) imgnum=imgnum-maxfiles;

  strcpy(leftfile, filelist[imgnum  ]->d_name);
  strcpy(rightfile,filelist[imgnum+1]->d_name);

  // convert leftfile & rightfile back to absolute path names
  sprintf(tmp2,"%s/%s",workingdir,leftfile);
  if(img_l.data!=0)   free(img_l.data);	// free buffer if it was used before
  img_l.data= read_JPEG_file(tmp2,  &img_l.width, &img_l.height);

  sprintf(tmp2,"%s/%s",workingdir,rightfile);
  if(img_r.data!=0)   free(img_r.data);	// free buffer if it was used before
  img_r.data= read_JPEG_file(tmp2, &img_r.width, &img_r.height);

  // display file names and size information in statusbar labels
  sprintf(tmp3,"%s  %ix%i",leftfile, img_l.width,img_l.height);
  gtk_label_set_text (lookup_widget(GTK_WIDGET(Main_window),"lb_leftfilename"), tmp3 );
  sprintf(tmp3,"%s  %ix%i",rightfile,img_r.width,img_r.height);
  gtk_label_set_text (lookup_widget(GTK_WIDGET(Main_window),"lb_rightfilename"), tmp3 ); 

  create_anaglyph();
  if(disp2_on==1)   display2();
  if(disp3_on==1)   display3();
 }
}


/******************************************************
 *  Function:  	void prev_pair(void);
 *
 *  Description: 
 *  - load the previous image pair 
 *      
*/
void prev_pair(void)
{
 unsigned int x;
 char tmp2[200];	// a temporary string
 char tmp3[200];	// a temporary string

 if(filelist==0) 
   printf("No Image loaded \n");
 else
 {
  imgnum=imgnum-2;
  if(imgnum<0) imgnum=imgnum+maxfiles;

  strcpy(leftfile, filelist[imgnum  ]->d_name);
  strcpy(rightfile,filelist[imgnum+1]->d_name);

  // convert leftfile & rightfile back to absolute path names
  sprintf(tmp2,"%s/%s",workingdir,leftfile);
  if(img_l.data!=0)   free(img_l.data);	// free buffer if it was used before
  img_l.data= read_JPEG_file(tmp2,  &img_l.width, &img_l.height);

  sprintf(tmp2,"%s/%s",workingdir,rightfile);
  if(img_r.data!=0)   free(img_r.data);	// free buffer if it was used before
  img_r.data= read_JPEG_file(tmp2, &img_r.width, &img_r.height);

  // display file names and size information in statusbar labels
  sprintf(tmp3,"%s  %ix%i",leftfile, img_l.width,img_l.height);
  gtk_label_set_text (lookup_widget(GTK_WIDGET(Main_window),"lb_leftfilename"), tmp3 );
  sprintf(tmp3,"%s  %ix%i",rightfile,img_r.width,img_r.height);
  gtk_label_set_text (lookup_widget(GTK_WIDGET(Main_window),"lb_rightfilename"), tmp3 ); 

  create_anaglyph();
  if(disp2_on==1)   display2();
  if(disp3_on==1)   display3();
 }
}



//*************************************************************************
//     T E S T   




void test(void)
{

}


